/*
 *  A libESMTP Example Application.
 *  Copyright (C) 2001,2002  Brian Stafford <brian@stafford.uklinux.net>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published
 *  by the Free Software Foundation; either version 2 of the License,
 *  or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* This program accepts a single file argument followed by a list of
   recipients.  The file is mailed to each of the recipients.

   Error checking is minimal to non-existent, this is just a quick
   and dirty program to give a feel for using libESMTP.
 */
#define _XOPEN_SOURCE 500

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include <stdarg.h>

#include <auth-client.h>
#include <libesmtp.h>

#if !defined (__GNUC__) || __GNUC__ < 2
# define __attribute__(x)
#endif
#define unused      __attribute__((unused))

const char * readline_cb_memory(void **buf, int *len, void *message_text);
int simple_send_mail(char *MTAHost, char *from, char *to, char *subject, char *mesaj);

/* returns status->code (250 when succefful) or 0 when an error occurs */
int simple_send_mail(char *MTAHost, char *from, char *to, char *subject, char *mesaj)
{
	smtp_session_t session;
	smtp_message_t message;
	smtp_recipient_t recipient;
	const smtp_status_t *status;
	struct sigaction sa;
	char buf[1500];
	char *mtaHost=(char *)malloc( strlen(MTAHost)+4 );
	enum notify_flags notify = Notify_NOTSET;
	/*notify |= Notify_SUCCESS | Notify_FAILURE | Notify_DELAY;*/
  
	session = smtp_create_session();
	message = smtp_add_message(session);

   /* NB.  libESMTP sets timeouts as it progresses through the protocol.
     In addition the remote server might close its socket on a timeout.
     Consequently libESMTP may sometimes try to write to a socket with
     no reader.  Ignore SIGPIPE, then the program doesn't get killed
     if/when this happens. */
	sa.sa_handler = SIG_IGN;
	sigemptyset(&sa.sa_mask);
	sa.sa_flags = 0;
	sigaction(SIGPIPE, &sa, NULL); 
	
	strcpy(mtaHost, MTAHost);
	strcat(mtaHost, ":25");
	if( !smtp_set_server(session, mtaHost) ) {
		puts("smptp_set_server: error !");
		free(mtaHost); return 0;
	}
	free(mtaHost);
	
	smtp_set_reverse_path(message, from);
	recipient = smtp_add_recipient(message, to);
	
	if( notify != Notify_NOTSET ) smtp_dsn_set_notify(recipient, notify);

	smtp_set_header(message, "To", NULL, to);
	smtp_set_header(message, "Subject", subject);
	smtp_set_header_option(message, "To", Hdr_OVERRIDE, 1);
	smtp_set_header_option(message, "From", Hdr_OVERRIDE, 1);
	smtp_set_header_option(message, "Subject", Hdr_OVERRIDE, 1);
	smtp_set_header_option(message, "Date", Hdr_OVERRIDE, 1);
	smtp_set_header_option(message, "Message-Id", Hdr_OVERRIDE, 1);
	smtp_set_header_option(message, "Return-Path", Hdr_OVERRIDE, 1);

	strcpy(buf,\
		"Subject: subject|"\
		"To: to@to.com|"\
		"From: from@from.com|"\
		"Message-Id: <mes-id>|"\
		"Date: 2004.10.10|");
	strcat(buf, mesaj); strcat(buf, "|");
	
	smtp_set_messagecb(message, readline_cb_memory, &buf);
		
  /* Initiate a connection to the SMTP server and transfer the message. */
	if( !smtp_start_session(session) ) {
		fprintf(stderr, "SMTP server problem: %s\n", smtp_strerror(smtp_errno(), buf, sizeof buf));
		/* Free resources consumed by the program. */
		smtp_destroy_session (session);
		return 0;
	}

	/* Report on the success or otherwise of the mail transfer. */
	status = smtp_message_transfer_status(message);
	
	/* Free resources consumed by the program. */
	smtp_destroy_session (session);
	return status->code;
}


/* Callback function to read the message from a file.  Since libESMTP
   does not provide callbacks which translate line endings, one must
   be provided by the application.

   The message is read a line at a time and the newlines converted
   to \r\n.  Unfortunately, RFC 822 states that bare \n and \r are
   acceptable in messages and that individually they do not constitute a
   line termination.  This requirement cannot be reconciled with storing
   messages with Unix line terminations.  RFC 2822 rescues this situation
   slightly by prohibiting lone \r and \n in messages.

   The following code cannot therefore work correctly in all situations.
   Furthermore it is very inefficient since it must search for the \n.
 */
const char * readline_cb_memory(void **buf, int *len, void *message_text)
{
	static unsigned int yer=0; /* will hold the current location in the string */
	char *mes=NULL;
	char *pstart;
	char *ps;
	int unsigned total_length=strlen(message_text);
  
	/* Copy the entire to a newly allocated memory, to be able to make some character cahnges
	like replacing | with NULL and inserting \r\n characters */
	mes=(char *)malloc( total_length+3 );
	strcpy(mes, message_text);
  
	/* printf("Total Message text: %s-\n", mes); */
  
	if( !len ) {
		/* puts("len=NULL, return"); */
		yer=0; free(mes); return NULL;
	}

	/* printf("Gelen len=%d, kalan mes[%d]:%s-\n", *len, yer, &mes[yer]); */
	
	if( yer>=total_length ) {
		/* puts("Mesaj bitti."); */
		*len=0; *buf=NULL;  	
		free(mes); return NULL;
    }
	
	pstart=mes; /* pstart holds the beginning address of the whole message string */
	ps=strchr(&mes[yer], '|'); ps[0]=(char)NULL; /* finish the string mes */
	ps++; /* ps will now point the next character after '\0' */
	/*printf("gonderilecek veri:%s-, sonraki kisim:%s, yer=%d\n", &mes[yer], ps, yer);*/
					
	strcpy(ps-1, "\r\n");
		
	*len = strlen(&mes[yer]);
	*buf=(char *)malloc( *len );
	if( !*buf ) {
		puts("Couldn't allocate memory !");
		free(mes); *len=0; return NULL;
	}
	strcpy( *buf, &mes[yer] );
    yer=(int)(ps-pstart); // find the length of the first part of the string using pointer addresses
	
	/*printf("yeni len=%d, buf=%s--\n", *len, (char *)*buf);*/
	free(mes); return *buf;
}
