/* Network Tracker
Copyright Ali BASEL
Ali BASEL <alib@sabanciuniv.edu>, <ali@basel.name.tr>
http://www.basel.name.tr
Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.
*/
#include <ping.h>
#include <sys/types.h>
#include <unistd.h>
#include <syslog.h>
#include <signal.h>
#include <time.h>
#include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <netdb.h>
#include <sys/socket.h> /* for AF_INET */

#include "tracker.h"

#ifdef LINUX
struct option longopts[] = {
    { "help", no_argument, NULL, 'h', },
    { "version", no_argument, NULL, 'v', },
    { "n", no_argument, NULL, 'n', },
    { NULL, 0, NULL, 0, }
};
#endif

#define TIME1 8
#define TIME2 16

#undef DEMO_USE_SNMP_VERSION_3

int sendmail(struct ayr *ayarlar);
int isle(struct ayr *ayarlar);
void usage(void);
void cik(int signo);
char *cevir(char *fin, char *address);

int ping_timeout=3;


void usage(void)
{
	printf("\nVersion: %s\n", VERSION);
	puts("Usage: trackerd                    Run as a daemon");
	puts("       trackerd -n                 Don't run as a daemon");
	puts("       trackerd -h, --help         This info");
	puts("       trackerd -v, --version      This info");
	puts("\nPlease see man trackerd(1) for more info or read the README file.");
	puts("\nAli BASEL <alib@sabanciuniv.edu>\nhttp://www.basel.name.tr\n");
	exit(0);
}

int main(int argc, char **argv)
{
	int	daemon = 1, pid, op;
	int	status, cevap_vermeyen_IP_sayisi=0, report=1, tracker_counter=-1;
	struct ayr ayarlar;
	time_t curtime;
	struct tm *tmb;
	char buf[MAXSTR];

#ifdef LINUX  
	while( (op=getopt_long(argc, argv, "n::hv", longopts, NULL)) != EOF ) {
#else
	while( (op=getopt(argc, argv, "n::hv")) != EOF ) {
#endif
		switch(op) {
			case 'n':
				daemon = 0;
				break;
			default: usage();
		}
	}
	
    puts("Look at syslog (facility LOCAL1) for more info !");
	openlog("trackerd", LOG_NOWAIT,  LOG_LOCAL1);

	// Reads sql hostname, username and password from CONFIG_FILE file
    if( !config_oku(&ayarlar) ) {
        printf("Please look at syslog message file for the error !\n\n");
        exit(0);
    }
         
    //ldap_uid_ekle(&ayarlar); exit(0);
    
	if(daemon) {
		if( (pid=fork())<0) {
			syslog(LOG_ERR, "main fork() error %m");
			//for child process pid=0, for main process pid=number
		}else if( pid ) exit(0); //main process ends, child is going to the background
	
		//No more output to the console is possible
		close( fileno(stdin) );
		close( fileno(stdout) );
		close( fileno(stderr) );	
		
	}

	(void)signal(SIGINT, cik);
	(void)signal(SIGTERM, cik);
	(void)signal(SIGHUP, cik);
	(void)signal(SIGQUIT, cik);
	

	syslog(LOG_INFO, "trackerd started");

	do {
		syslog(LOG_INFO, "trackerd has started to check devices");
		
		status=IPListAlYaz(&ayarlar);
		if( !status ) {
			syslog(LOG_INFO, "main: Couldn't read the IP addresses !");
			syslog(LOG_INFO, "main: Sleeping for %d seconds !", ayarlar.Period);
            sleep(ayarlar.Period); continue;

		}

		// Database'den ayarlari okuyup structure'a yaz.
		// Cevap vermeyen cihaz oldugunda gonderilecek mail ile ilgili bilgiler
		// aliniyor.
		if( !Ayarlar(&ayarlar) ) {
			syslog(LOG_INFO, "main: Couldn't open the database, for reading the settings !");
       		syslog(LOG_INFO, "main: Sleeping for %d seconds !", ayarlar.Period);
            sleep(ayarlar.Period); continue;
		}
		
		// Database'den CEVAPYOK dosyasina alinmis olan IP'ler ping ve snmp ile
		// kontrol edilecek; cevap vermeyen olursa e-mail ile bildirilecek.
		if( isle(&ayarlar) ) cevap_vermeyen_IP_sayisi=sendmail( &ayarlar );

        if( ++tracker_counter==ayarlar.TrackerRate || tracker_counter==0 ) {
            tracker_counter=0;

            if( (pid=fork())<0) {
                syslog(LOG_ERR, "main: tracking process fork() error %m");
                // for child process pid=0, for main process pid=number
            }
            else
                if( pid ) { 
                    (void)signal(SIGINT, cik);
                    (void)signal(SIGTERM, cik);
                    (void)signal(SIGHUP, cik);
                    (void)signal(SIGQUIT, cik);
                    tracker(&ayarlar);
                    exit(0);
                }
        }

		// Belirli saatlerde programin problemsiz olarak calistigi bildirilecek.
		time(&curtime); tmb=localtime(&curtime);
		if( (tmb->tm_hour==TIME1 || tmb->tm_hour==TIME2) && report ) {
			sprintf(buf, "trackerd is running. Total checked IP number: %d.\n"\
						"These IPs have been checked by snmp and ping.\n"\
						"Total Unreachable IP addr. number: %d\n"\
						"Last MAC database update time: %s", status, cevap_vermeyen_IP_sayisi, ayarlar.LastUpdateTime);
            syslog(LOG_INFO, buf);
            syslog(LOG_INFO, "MTAHost:%s, From:%s, to:%s", ayarlar.MTAHost, ayarlar.FromName, ayarlar.To1);
            simple_send_mail(ayarlar.MTAHost, ayarlar.FromName, ayarlar.To1, "trackerd OK", buf);
			report=0; //Do not send this report again
			cevap_vermeyen_IP_sayisi=0;
		}
		
		// You may send the report if ...
		if( !report && (tmb->tm_hour != TIME1  && tmb->tm_hour != TIME2)  ) report=1;
 		
		syslog(LOG_INFO, "main: Sleeping for %d seconds", ayarlar.Period);
        sleep(ayarlar.Period);
	} while (daemon);
	
	exit(0);
}

int isle(struct ayr *ayarlar)
{
    int ipno=0, cevap, error=0;
    static char line[256], host[256];
	FILE *cevapvar;
	FILE *cevapyok;
	FILE *listeyaz;

	cevapvar=fopen(CEVAPVAR, "w");
	cevapyok=fopen(CEVAPYOK, "w");
	listeyaz=fopen(LISTE, "r");

	while(fgets(line,132,listeyaz)) {
    	if (sscanf(line,"%s",host) != 1) continue;
        if ((!*host) || (host[0]=='#')) continue; /* magic to avoid comments */
		ipno++;
		cevap=pingthost( host, ping_timeout );
	    if( cevap>0 ) fprintf(cevapvar, "%s is alive\n", host);
        else if( !snmp(host, ayarlar) ) { // snmp'den de cevap gelmediyse yaz
			fprintf(cevapyok, "%s\n", host);
			error=1;
		}
		syslog(LOG_INFO, "isle: host[%d]:%s, cevap=%d", ipno, host, cevap);
	}
	fclose(cevapvar);
	fclose(cevapyok);
	fclose(listeyaz);
	
	return error;
}
 
int sendmail(struct ayr *ayarlar)
{
  	char mesaj[1024], buf[255], ipaddress[4], *dns;
  	int i=0, num=0;
	FILE *cevapyok=fopen(CEVAPYOK, "r");
	mesaj[0]=(char)NULL;
	

	while( !feof(cevapyok) ) {
	  	buf[i]=(char)fgetc( cevapyok );
		if( buf[i++]=='\n' ) {
            		num++; // Adds DNS name of the IP address to the mesaj string
			dns=cevir(ipaddress, buf);
			syslog(LOG_INFO,"dns=%s\n", dns);
			strcat( mesaj, "\n");
			strcat( mesaj, dns );
			i=0;
            		if( strlen(mesaj) > 900 ) {
                		strcat( mesaj, " Too many devices...");
                		break;
            		}
        	}
	}
	
	fclose(cevapyok);

	if( strlen(mesaj)<5 ) {
		syslog(LOG_INFO, "sendmail:cevap vermeyen cihaz yok\n");
		return 0;
	}
	
	syslog(LOG_INFO, "sendmail: Cevap vermeyen cihazlar:\n%s\n", mesaj);
	syslog(LOG_INFO, "sendmail:from:%s, kime:%s, MTAHost:%s, Subj:%s\n", ayarlar->FromName, ayarlar->To1, ayarlar->MTAHost, ayarlar->Subject);
	simple_send_mail(ayarlar->MTAHost, ayarlar->FromName, ayarlar->To1, ayarlar->Subject, mesaj);
	return num;
}

char *cevir(char *fin, char *address)
{
	struct hostent *host;
	char buf[32];
	static char sonuc[255];
	int i = 0, j = 0, num = 0, exit=0;
	        
        do {
		if( (*(address + i) == '.') || (*(address + i) == (char)NULL) ) {
			buf[j]= (char)NULL;
			fin[num]=atoi(buf);
			//printf("buf=%s, fin[%d]=%d\n", buf, num, fin[num]);
			num++; j=-1; 
			if( *(address + i) == (char)NULL ) exit=1;
		}
		else buf[j] = *(address + i);
		++i; j++;
	}while( !exit );
	host=gethostbyaddr(fin, 4, AF_INET);
	if( !host ) {
	 // Coldn't resolv
    		strncpy(sonuc, address, 254);
  	}
  	else strncpy(sonuc, host->h_name, 254);
	return sonuc;
}


void cik(int signo)
{
	syslog(LOG_INFO, "cik: trackerd ended.");
	closelog();
	exit(1);
}

int snmp(char *host, struct ayr *ayarlar)
{
    struct snmp_session session, *ss;
    struct snmp_pdu *pdu;
    struct snmp_pdu *response;

    oid anOID[MAX_OID_LEN];
    size_t anOID_len = MAX_OID_LEN;

    int status;
    
    /*
     * Initialize the SNMP library
     */
    init_snmp("snmpapp");

    /*
     * Initialize a "session" that defines who we're going to talk to
     */
    snmp_sess_init( &session ); /* set up defaults */
    session.peername = strdup(host);

    /* set the SNMP version number */
    session.version = SNMP_VERSION_1;

    /* set the SNMPv1 community name used for authentication */
    session.community = ayarlar->SnmpRO;
    session.community_len = strlen(session.community);

    /*
     * Open the session
     */
    SOCK_STARTUP;
    ss = snmp_open(&session); /* establish the session */

    if (!ss) {
        snmp_perror("ack");
        snmp_log(LOG_ERR, "something horrible happened!!!\n");
        return 0;
    }
    
    /*
     * Create the PDU for the data for our request.
     *   1) We're going to GET the system.sysDescr.0 node.
     */
    pdu = snmp_pdu_create(SNMP_MSG_GET);
    read_objid(".1.3.6.1.2.1.1.1.0", anOID, &anOID_len);

#if OTHER_METHODS
    get_node("sysDescr.0", anOID, &anOID_len);
    read_objid("system.sysDescr.0", anOID, &anOID_len);
#endif

    snmp_add_null_var(pdu, anOID, anOID_len);
  
    /*
     * Send the Request out.
     */
    status = snmp_synch_response(ss, pdu, &response);

    if (response)
      snmp_free_pdu(response);
    snmp_close(ss);

    SOCK_CLEANUP;
    return(status==STAT_SUCCESS ? 1 : 0);
}
