/* Network Tracker
Copyright Ali BASEL
Ali BASEL <alib@sabanciuniv.edu>, <ali@basel.name.tr>
http://www.basel.name.tr
Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "trackerdbgui.h"
#include "interface.h"
#include "support.h"

void on_search_field_edited(GtkCellRendererText *celltext, const gchar *string_path, const gchar *new_text, gpointer data)
{
	GtkTreeView *gtktreeview=(GtkTreeView *)data;
	GtkTreeModel *model= gtk_tree_view_get_model(gtktreeview);
	GtkTreeIter iter;
	gchar *oldval=NULL;
	GtkTreePath *path;
	GtkTreeViewColumn *focus_column;
	int selected_col_number=-1;
	
	ldebug(2, "on_search_field_edited: entered, path=%s\n", string_path);
	
	gtk_tree_view_get_cursor(gtktreeview, &path, &focus_column);
	selected_col_number= GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(focus_column), "columnnum"));
	ldebug(2, "on_search_field_edited: selected_col_number=%d\n", selected_col_number);
	
	gtk_tree_model_get_iter_from_string(model, &iter, string_path);
	gtk_tree_model_get(model, &iter, selected_col_number, &oldval,-1);
	g_print("on_search_field_edited: Selected Col#=%d, Old value:%s, new value:%s-\n", selected_col_number, oldval, new_text);
	gtk_list_store_set(GTK_LIST_STORE(model), &iter, selected_col_number, new_text, -1);
		
	g_free(oldval);
	gtk_tree_path_free(path);	
}

void on_searchwindow_search_clicked(GtkButton *button, gpointer user_data)
{
	ldebug(2, "on_searchwindow_search_clicked\n");
	gtk_widget_hide(user_data);
	
}

void on_searchwindow_cancel_clicked(GtkButton *button, gpointer user_data)
{
	ldebug(2, "on_searchwindow_search_clicked\n");
	gtk_widget_destroy(user_data);
}


void on_edit_search(GtkMenuItem *menuitem, gpointer user_data)
{
    Params *params=(Params *)user_data;
	TabInfo *ti=params->last_selected_gtktreeview;
	
	GtkWidget *search_win;
	GtkTreeView *gtktreeview;
	GtkTreeViewColumn *column;
	GtkCellRenderer *renderer;
	GtkListStore *store;
	GtkTreeIter iter;
	int i;
	
	ldebug(2,"on_edit_search called: Table:%s\n", ti->table);
	
/*	
	if( gtk_toggle_button_get_active( (GtkToggleButton*)GTK_WIDGET(lookup_widget(((Params *)user_data)->mainwindow, "search")) ) ) {
		ldebug(2, "TRUE\n");
		//newSearchWindow( ((Params *)user_data)->TabInfoStructs[MAIN] );
	}
	// Cancel the search filter, show all records
	else on_showrecords_clicked( button, user_data );
*/	
	
	search_win=create_searchwindow();
	gtk_widget_show(search_win);
	
	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)search_win,"searchbutton")), "clicked", G_CALLBACK(on_searchwindow_search_clicked), search_win);

	g_signal_connect( GTK_OBJECT(lookup_widget((GtkWidget *)search_win,"cancelbutton")), "clicked", G_CALLBACK(on_searchwindow_cancel_clicked), search_win);

	gtktreeview=(GtkTreeView *)GTK_WIDGET(lookup_widget(search_win, "searchgtktreeview"));
	renderer = gtk_cell_renderer_text_new();
	/* gtktreeview icindeki gtkliststore grid'inin editable olmasi saglaniyor */
	g_object_set(renderer, "editable", TRUE, NULL);
	g_signal_connect(renderer, "edited", G_CALLBACK(on_search_field_edited), gtktreeview);

	for(i=0; i < ti->nrcols; i++) {
		gtk_tree_view_insert_column_with_attributes(gtktreeview, -1, ti->titles[i], renderer, "text",  i, NULL);
		column=gtk_tree_view_get_column(gtktreeview, i);
		gtk_tree_view_column_set_sort_column_id(column, i);
		/* Daha sonra gtktreeview'da bir yere secildigi zaman
		column number'i bulmak boyle bir cift tanimlaniyor */
		g_object_set_data(G_OBJECT(column), "columnnum", GUINT_TO_POINTER(i));
	}

	// gtktreeview'da gosterilmek uzere bir gtk_list_store olusturulmaya baslaniyor
	// gtk_list_store'un field'larinin sayisi ve tipleri tanimlaniyor
	store = gtk_list_store_newv(ti->numflds, ti->field_types_for_gtk_list_store);
	gtk_list_store_append(store, &iter);
	
	
	/* Clear the old gtk_list_store */
	//gtk_list_store_clear( (GtkListStore *)gtk_tree_view_get_model(gtktreeview) );
  	
	/* Then, set the new gtk_list_store */
	gtk_tree_view_set_model(gtktreeview, GTK_TREE_MODEL(store));
  	g_object_unref(store); // destroy model automatically with view
}

void on_edit_delete(GtkMenuItem *menuitem, gpointer user_data)
{
	Params *params=(Params *)user_data;
	GtkTreeSelection *gtktreeselection=NULL;
	GtkTreeModel *model;
	GtkTreeIter selected_row;
	gchar *master_field_value=NULL, *sortkey_value=NULL;
	TabInfo *ti=params->last_selected_gtktreeview;
	TabInfo *ti_detail;
	char query[MAXQUERY_LENGTH+1];
	int del_master_rec_nr=-1, del_detail_rec_nr=-1;
	
	ldebug(2,"on_edit_delete called\n");
	gtktreeselection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ti->gtktreeview));
	
	if( !gtk_tree_selection_count_selected_rows(gtktreeselection) ) {
		ShowMessageBox("Please select a record !", params);
		return;
	}
	
	if( GTK_RESPONSE_REJECT==ShowConfirmBox("Record will be deleted ?", params) ) return;
	
	gtk_tree_selection_get_selected(gtktreeselection, &model, &selected_row);
	
	/* ti->master_field_no  tablodaki master field'in sira numarasini gosteriyor */
	/* secilen satirdaki bu deger master_field_value alanina okunuyor */
	gtk_tree_model_get(model, &selected_row, ti->sortkey, &sortkey_value, -1);
	gtk_tree_model_get(model, &selected_row, ti->master_field_no, &master_field_value, -1);
	ldebug(2, "on_edit_delete_activate: sortkey_value=%s, table:%s\n", sortkey_value, ti->table);
	
	sprintf(query, "delete from %s where %s=", ti->table, ti->fields[ti->sortkey].name);
	strncat(query, addStringQuotes(ti->fields[ti->sortkey].type), MAXQUERY_LENGTH-strlen(query));
	strncat(query, sortkey_value, MAXQUERY_LENGTH-strlen(query));
	strncat(query, addStringQuotes(ti->fields[ti->sortkey].type), MAXQUERY_LENGTH-strlen(query));
		
	ldebug(2, "on_edit_delete_activate: query:%s\n", query);
	if( !executeQuery(params, query) ) /* There isn't any error, record has been removed */
		gtk_list_store_remove((GtkListStore *)model, &selected_row); /* Remove that row also*/
	
	del_master_rec_nr=mysql_affected_rows(params->dbsock);
	
	if( ti->DetailTableNo > 0 ) {
		ldebug(2, "on_edit_delete: deleting detail records...\n");
		ti_detail=params->TabInfoStructs[ti->DetailTableNo];
		sprintf(query, "delete from %s where %s=", ti_detail->table, ti->master_field_name);
		strncat(query, addStringQuotes(ti->fields[ti->master_field_no].type), MAXQUERY_LENGTH-strlen(query));
		strncat(query, master_field_value, MAXQUERY_LENGTH-strlen(query));
		strncat(query, addStringQuotes(ti->fields[ti->master_field_no].type), MAXQUERY_LENGTH-strlen(query));
		ldebug(2, "on_edit_delete_activate: query:%s\n", query);
		if( !executeQuery(params, query) ) {/* There isn't any error, record has been removed */
			del_detail_rec_nr=mysql_affected_rows(params->dbsock);
			showDetails(ti, master_field_value);
		}
		update_statusbar(params, "Master Table: %s, Deleted record number=%d, Detail table:%s, Deleted record number=%d", ti->table, del_master_rec_nr, ti_detail->table, del_detail_rec_nr);
	}
	else update_statusbar(params, "Table: %s, Deleted record number=%d", ti->table, del_master_rec_nr);

   	if(sortkey_value) g_free(sortkey_value); 
	if(master_field_value) g_free(master_field_value);
}


void on_edit_insert(GtkMenuItem *menuitem, gpointer user_data)
{
	Params *params=(Params *)user_data;
	TabInfo *ti=params->last_selected_gtktreeview;
	GtkTreeModel *model;
	GtkTreeIter iter;
	GtkTreePath *path;
	GtkTreeViewColumn *focus_column;
	int i;
	char query[MAXQUERY_LENGTH+1];
	
	ldebug(2,"on_edit_insert_activate called\n");
	
	model=gtk_tree_view_get_model(ti->gtktreeview);
	gtk_list_store_append( (GtkListStore *)model, &iter);
	for(i=0; i < ti->nrcols; i++) {
		ldebug(2,"on_edit_insert_activate: default value in SQL for column[%d]:%s\n", i, ti->fields[i].def);
		gtk_list_store_set( (GtkListStore *)model, &iter, i, ti->fields[i].def,-1);
	}
	path=gtk_tree_model_get_path(model, &iter);
	focus_column=gtk_tree_view_get_column(ti->gtktreeview, 0);
	gtk_tree_view_set_cursor(ti->gtktreeview, path, focus_column, FALSE);	
	gtk_widget_grab_focus((GtkWidget *)ti->gtktreeview);
	
	
	sprintf(query, "insert into %s values(", ti->table);
	for(i=0; i<ti->numflds; i++) {
		strncat(query, addStringQuotes(ti->fields[i].type), MAXQUERY_LENGTH-strlen(query));
		strncat(query, ti->fields[i].def, MAXQUERY_LENGTH-strlen(query));
		strncat(query, addStringQuotes(ti->fields[i].type), MAXQUERY_LENGTH-strlen(query));
		strncat(query, ",", MAXQUERY_LENGTH-strlen(query));
	}
	query[strlen(query)-1]=(char)NULL; /* removes the last ',' */
	strncat(query, " )", MAXQUERY_LENGTH-strlen(query));
	
	ldebug(2, "on_edit_insert_activate: query:%s\n", query);
	if( executeQuery(params, query) ) /* There is an error */
		gtk_list_store_remove((GtkListStore *)model, &iter); /* Remove that row */
	ldebug(2,"on_edit_insert_activate son \n");   
}


void on_edit_show(GtkMenuItem *menuitem, gpointer par)
{
    Params *params=(Params *)par;
	TabInfo *ti=params->last_selected_gtktreeview;
    ldebug(2, "on_showrecords_clicked: table name=%s\n", ti->table);
	
    showRecords( ti );
}

