#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "trackerdbgui.h"

#include "interface.h"
#include "support.h"

void debug(char *format, ...)
{
#ifdef DEBUG
   va_list ap;
   
   va_start(ap,format);
   vfprintf(stderr,format, ap);
   va_end(ap);
#endif
}

void ldebug(int level, char *format, ...)
{
#ifdef DEBUG
   va_list ap;
   
   if (DEBUGLEVEL < level)
     return;
   va_start(ap,format);
   vfprintf(stderr,format, ap);
   va_end(ap);
#endif
}

int ShowConfirmBox(char *message, Params *params)
{
	GtkWidget *dialogtext;
	GtkDialog *dialogbox;
	int result;
	 
	dialogbox = (GtkDialog *)gtk_dialog_new_with_buttons("Trackerdbgui", params->mainwindow,
				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
				GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, NULL);
	
	dialogtext = gtk_label_new(message);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialogbox)->vbox), dialogtext);
	gtk_label_set_justify(GTK_LABEL(dialogtext), GTK_JUSTIFY_CENTER);
	gtk_label_set_line_wrap(GTK_LABEL(dialogtext), TRUE);
	gtk_widget_show_all((GtkWidget *)dialogbox);								
	
	result = gtk_dialog_run(dialogbox);
	gtk_widget_destroy((GtkWidget *)dialogbox);
	return result;
}

void ShowMessageBox(char *message, Params *params)
{
	GtkWidget *dialogtext;
	GtkDialog *dialogbox;
	 
	dialogbox = (GtkDialog *)gtk_dialog_new_with_buttons("trackerdbgui", params->mainwindow,
				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
				GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
				NULL);
	
	dialogtext = gtk_label_new(message);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialogbox)->vbox), dialogtext);
	//gtk_label_set_justify(GTK_LABEL(dialogtext), GTK_JUSTIFY_CENTER);
	gtk_label_set_line_wrap(GTK_LABEL(dialogtext), TRUE);
	gtk_widget_show_all((GtkWidget *)dialogbox);								
	
	gtk_dialog_run(dialogbox);
	gtk_widget_destroy((GtkWidget *)dialogbox);
}


void showSqlError(Params *par, char *s, ...)
{
   va_list ap;
   char error[MAXMESSAGE];
   
   va_start(ap,s);
   vsnprintf(error,sizeof(error) - 1, s, ap);
   
   ldebug(2,"SQL Error: %s, %s\n",error, sql_error(par->dbsock));
   showVarArgMessage(par, "SQL Error: %s, %s\n",error, sql_error(par->dbsock));
}

void showVarArgMessage(Params *par, char *s,...)
{
   va_list ap;
   char message[MAXMESSAGE];
   
   va_start(ap,s);
   vsnprintf(message,sizeof(message) - 1, s, ap);
   ldebug(2,"Message: %s\n",message);
   
   ShowMessageBox(message, par);
}
