#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "trackerdbgui.h"

#include "interface.h"
#include "support.h"

void update_statusbar(Params *par, char *format, ...)
{ 
	char message[MAXMESSAGE];
	va_list ap;
	GtkStatusbar *gtkstatusbar = (GtkStatusbar *)GTK_WIDGET(lookup_widget((GtkWidget *)par->mainwindow, "statusbar"));
	//gtk_statusbar_pop(gtkstatusbar, 0); /* clear any previous message, underflow is allowed */
   
	va_start(ap,format);
	vsnprintf(message, sizeof(message)-1, format, ap);
	va_end(ap);
	gtk_statusbar_push(gtkstatusbar, 0, message);
}

void on_DetailsGtkCheckButton_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
	Params *par=(Params *)user_data;
    TabInfo *ti=par->last_selected_gtktreeview;
	ti->show_details=gtk_toggle_button_get_active( togglebutton );
}


gboolean gtktreeview_onButtonPressed(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	/* When pressed two times on the ctrl key, show details*/
	#define CTRL_KEY 37
	TabInfo *ti=(TabInfo*)user_data;
	Params *par=ti->params;
	static int old_key=1;
	ldebug(2, "on_key pressed: hardware_keycode=%d\n", event->hardware_keycode);
	ldebug(2, "old_key=%d, newkey=%d\n",old_key, event->hardware_keycode); 
	if( ti->DetailTableNo==-1 ) {
		ldebug(2, "gtktreeview_onButtonPressed: table:%s has no master-detail relation\n", ti->table);
		update_statusbar(ti->params, "Table: %s has no master-detail relation !", ti->table);
		return FALSE;
	}
	/* ctrl tusuna pes pese iki kere basildiysa details'in gosterilmesi enable/disable ediliyor */
	if( old_key==CTRL_KEY && event->hardware_keycode==CTRL_KEY) {
		if( ti->show_details ) {
			ti->show_details=0;
			gtk_toggle_button_set_active( (GtkToggleButton*)GTK_WIDGET(lookup_widget((GtkWidget *)par->mainwindow, "DetailsGtkCheckButton")), FALSE );
			gtk_widget_hide( ((Params *)(ti->params))->detailframe );
		} 
		else {
			ti->show_details=1;
			gtk_toggle_button_set_active( (GtkToggleButton*)GTK_WIDGET(lookup_widget((GtkWidget *)par->mainwindow, "DetailsGtkCheckButton")), TRUE );
			/* Ayni tusa iki kere basildi ve details'in gozukmesi isteniyor
			yukari/asagi ok tuslarina basilmasini beklemeden details'i goster */
			record_selected(ti->gtktreeview, ti);	
		}	
		
	}
	old_key=event->hardware_keycode;
	if( event->hardware_keycode==106 ) insert_key_pressed(ti);
	
	/* return FALSE ile "button_pressed" ile ilgili diger eventler cagiriliyor*/
	/* scroll vs. yapiliyor */
	return FALSE;
}


/* If we come here, then the user wants to insert a detail record*/
int insert_key_pressed(TabInfo *tim)
{
	GtkTreeSelection *gtktreeselection;
	GtkTreeModel *model;
	GtkTreeIter selected_row;
	char *master_field_value=NULL;
	int is_visible=-1;
	
	TabInfo *ti_detail=NULL;
	TabInfo *ti_master=((Params *)(tim->params))->last_selected_gtktreeview;
	//GtkListStore *store;
	GtkTreeIter iter;
	GtkTreePath *path;
	GtkTreeViewColumn *focus_column;
	int i;
	char query[MAXQUERY_LENGTH-1];
		
	ldebug(2,"insert_key_pressed function entered\n");
	
	if( ti_master->DetailTableNo == -2 ) {
		ldebug(2, "insert_key_pressed: %s table is a detail of another table.\n", ti_master->table);
		return TRUE;
	}	
	
	if( ti_master->DetailTableNo == -1 ) {
		ldebug(2, "insert_key_pressed: %s table has no master-detail relation\n", ti_master->table);
		gtk_widget_hide( ((Params *)(ti_master->params))->detailframe );
		return TRUE;
	}	
	
	ti_detail=((Params *)(ti_master->params))->TabInfoStructs[ti_master->DetailTableNo];	
	
	/* API'de gorulen "property"'lerin okunmasina ornek */
	g_object_get(G_OBJECT(((Params *)(ti_master->params))->detailframe), "visible", &is_visible, NULL);
	ldebug(2, "insert_key_pressed: detailframe is visible:%d\n", is_visible);

	if( !is_visible ) {	
		/* If the detail frame is already visible no need to make it visible again */
		gtk_widget_show( ((Params *)(ti_master->params))->detailframe );
	}	
	
	gtktreeselection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ti_master->gtktreeview));
	gtk_tree_selection_get_selected(gtktreeselection, &model, &selected_row);
	
	/* ti_master->master_field_no  tablodaki master field'in sira numarasini gosteriyor */
	/* secilen satirdaki bu deger master_field_value alanina okunuyor */
	gtk_tree_model_get(model, &selected_row, ti_master->master_field_no, &master_field_value, -1);
	ldebug(2, "insert_key_pressed: master_field_value=%s\n", master_field_value);
	
	model=gtk_tree_view_get_model(ti_detail->gtktreeview);
	gtk_list_store_append( (GtkListStore *)model, &iter);
	for(i=0; i < ti_detail->nrcols; i++) {
		if( !strcmp(ti_detail->fields[i].name, ti_master->master_field_name) ) { 
			gtk_list_store_set( (GtkListStore *)model, &iter, i, master_field_value,-1);
			ldebug(2,"insert_key_pressed: value:%s\n", master_field_value);
		}
		else gtk_list_store_set( (GtkListStore *)model, &iter, i, ti_detail->fields[i].def,-1);
	}
	path=gtk_tree_model_get_path(model, &iter);
	focus_column=gtk_tree_view_get_column(ti_detail->gtktreeview, 0);
	gtk_tree_view_set_cursor(ti_detail->gtktreeview, path, focus_column, FALSE);	
	gtk_widget_grab_focus((GtkWidget *)ti_detail->gtktreeview);
	
	
	sprintf(query, "insert into %s values(", ti_detail->table);
	for(i=0; i<ti_detail->numflds; i++) {
		strncat(query, addStringQuotes(ti_detail->fields[i].type), MAXQUERY_LENGTH-strlen(query));
		if( !strcmp(ti_detail->fields[i].name, ti_master->master_field_name) ) 
			strncat(query, master_field_value, MAXQUERY_LENGTH-strlen(query));
		else
			strncat(query, ti_detail->fields[i].def, MAXQUERY_LENGTH-strlen(query));
		strncat(query, addStringQuotes(ti_detail->fields[i].type), MAXQUERY_LENGTH-strlen(query));
		strncat(query, ",", MAXQUERY_LENGTH-strlen(query));
	}
	query[strlen(query)-1]=(char)NULL; /* removes the last ',' */
	strncat(query, " )", MAXQUERY_LENGTH-strlen(query));
		
	
	ldebug(2, "insert_key_pressed: query:%s\n", query);
	if( executeQuery((Params *)(ti_master->params), query) ) /* There is an error */
		gtk_list_store_remove((GtkListStore *)model, &iter); /* Remove that row */

	// shows details of the selected record
	showDetails(ti_master, master_field_value);
   	if( master_field_value ) g_free(master_field_value);
	return FALSE;
}

/* If we come here, then the user has selected a row*/
int record_selected(GtkTreeView *treeview, gpointer user_data)
{
	GtkTreeSelection *gtktreeselection;
	GtkTreeModel *model;
	GtkTreeIter selected_row;
	gchar *master_field_value;
	TabInfo *ti = (TabInfo *)user_data;
	int is_visible=-1;
	
	ldebug(2,"record_selected: You selected a row\n");
	((Params *)(ti->params))->last_selected_gtktreeview=ti;
	if( ti->DetailTableNo == -2 ) {
		ldebug(2, "%s table is a detail of another table.\n", ti->table);
		return TRUE;
	}	
	
	if( ti->DetailTableNo == -1 ) {
		ldebug(2, "%s table has no master-detail relation\n", ti->table);
		gtk_widget_hide( ((Params *)(ti->params))->detailframe );
		return TRUE;
	}	
	
	if( !ti->show_details ) {
		ldebug(2, "show_details is not enabled, so don't show the details\n");
		//gtk_tree_view_set_model((getParams()->TabInfoStructs[ti->DetailTableNo])->gtktreeview, NULL);
		gtk_widget_hide( ((Params *)(ti->params))->detailframe );
		return TRUE;
	}
	
	/* API'de gorulen "property"'lerin okunmasina ornek */
	g_object_get(G_OBJECT(((Params *)(ti->params))->detailframe), "visible", &is_visible, NULL);
	ldebug(2, "record_selected: detailframe is visible:%d\n", is_visible);

	if( !is_visible ) {	
		/* If the detail frame is already visible no need to make it visible again */
		gtk_widget_show( ((Params *)(ti->params))->detailframe );
	}	
	
	gtktreeselection = gtk_tree_view_get_selection(GTK_TREE_VIEW(ti->gtktreeview));
	gtk_tree_selection_get_selected(gtktreeselection, &model, &selected_row);
	
	/* ti->master_field_no  tablodaki master field'in sira numarasini gosteriyor */
	/* secilen satirdaki bu deger master_field_value alanina okunuyor */
	gtk_tree_model_get(model, &selected_row, ti->master_field_no, &master_field_value, -1);
	ldebug(2, "master_field_value=%s\n", master_field_value);
	
	// shows details of the selected record
	showDetails(ti, master_field_value);
   	g_free(master_field_value);
	return FALSE;
}

void on_field_edited(GtkCellRendererText *celltext, const gchar *string_path, const gchar *new_text, gpointer data)
{
	TabInfo *ti=(TabInfo *)data;
	GtkTreeModel *model= gtk_tree_view_get_model(ti->gtktreeview);
	GtkTreeIter iter;
	gchar *oldval=NULL, *master_field_value;
	GtkTreePath *path;
	GtkTreeViewColumn *focus_column;
	int selected_col_number=-1;
	char query[MAXQUERY_LENGTH-1];
	
	ldebug(2, "on_field_edited: entered, path=%s\n", string_path);
	
	gtk_tree_view_get_cursor(ti->gtktreeview, &path, &focus_column);
	selected_col_number= GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(focus_column), "columnnum"));
	ldebug(2, "on_field_edited: selected_col_number=%d\n", selected_col_number);
	
	gtk_tree_model_get_iter_from_string(model, &iter, string_path);
	gtk_tree_model_get(model, &iter, selected_col_number, &oldval,-1);
	gtk_tree_model_get(model, &iter, ti->sortkey, &master_field_value,-1);
	g_print("on_field_edited: Selected Col#=%d, Old value:%s, new value:%s-\n", selected_col_number, oldval, new_text);
	
	
	sprintf(query, "update %s set %s=", ti->table, ti->fields[selected_col_number].name);
	strncat(query, addStringQuotes(ti->fields[selected_col_number].type), MAXQUERY_LENGTH-strlen(query));
	strncat(query, new_text, MAXQUERY_LENGTH-strlen(query));
	strncat(query, addStringQuotes(ti->fields[selected_col_number].type), MAXQUERY_LENGTH-strlen(query));
	strncat(query, " where ", MAXQUERY_LENGTH-strlen(query));
	strncat(query, ti->fields[ti->sortkey].name, MAXQUERY_LENGTH-strlen(query));
	strncat(query, "=", MAXQUERY_LENGTH-strlen(query) );
	strncat(query, addStringQuotes(ti->fields[ti->sortkey].type), MAXQUERY_LENGTH-strlen(query));
	strncat(query, master_field_value, MAXQUERY_LENGTH-strlen(query) );
	strncat(query, addStringQuotes(ti->fields[ti->sortkey].type), MAXQUERY_LENGTH-strlen(query));
	
	ldebug(2, "on_field_edited: query:%s\n", query);
	if( GTK_RESPONSE_ACCEPT==ShowConfirmBox("  Update the record ?  ", (Params *)(ti->params)) ) {
		if( !executeQuery((Params *)(ti->params), query) ) /* No error */
			gtk_list_store_set(GTK_LIST_STORE(model), &iter, selected_col_number, new_text, -1);
	}
		
	g_free(oldval); g_free(master_field_value);
	//gtk_tree_view_set_cursor(ti->gtktreeview, path, focus_column, FALSE);
	gtk_widget_grab_focus((GtkWidget*)ti->gtktreeview);
	gtk_tree_path_free(path);	
}


void showRecords(TabInfo *ti)
{
	int x, i;
	char query[MAXQUERY], buf[255];
	SQL_ROW row;
	SQL_RES *res;
	GtkListStore  *store;
	GtkTreeIter    iter;

	ldebug(2,"showRecords entered, table:%s\n", ti->table);	
	
	snprintf(query,sizeof(query) - 1,SELECT ORDERBY, ti->table);
	strcat(query,ti->fields[ti->sortkey].name);
	ldebug(2,"showRecords: query:%s-\n", query);
	if( executeQuery( (Params *)(ti->params), query) ) {
		showSqlError( (Params *)(ti->params),"showRecords: Query in showRecords failed !");
		return;
	}
	res = sql_store_result( ((Params *)(ti->params))->dbsock);

	// gtktreeview'da gosterilmek uzere bir gtk_list_store olusturulmaya baslaniyor
	// gtk_list_store'un field'larinin sayisi ve tipleri tanimlaniyor
	store = gtk_list_store_newv(ti->numflds, ti->field_types_for_gtk_list_store);
	
	ti->nrrows=sql_num_rows(res);
	ldebug(2, "showRecords: nrrows=%d\n", ti->nrrows);
	update_statusbar((Params *)(ti->params), "Table:%s, Number of Records=%d", ti->table, ti->nrrows);
	for(x=0; x < ti->nrrows; x++) {
		if( (row = sql_fetch_row(res)) == NULL) {
			showSqlError( (Params *)(ti->params),"showRecords: Error reading record %d !",x);
			break;
		}
		gtk_list_store_append(store, &iter);
		for(i=0; i < ti->nrcols; i++) {
			if( ti->field_types_for_gtk_list_store[i]==G_TYPE_INT ) {
				sprintf(buf, "%d", (int)row[i]);
				gtk_list_store_set(store, &iter, i, buf,-1);
			}
			else gtk_list_store_set(store, &iter, i, row[i],-1);
		}
	}
	free(res);
	
	/* Clear the old gtk_list_store */
	gtk_list_store_clear( (GtkListStore *)gtk_tree_view_get_model(ti->gtktreeview) );
  	
	/* Then, set the new gtk_list_store */
	gtk_tree_view_set_model(ti->gtktreeview, GTK_TREE_MODEL(store));
  	g_object_unref(store); // destroy model automatically with view
	
	/* Show all the fields of the detail table */
	if( ti->DetailTableNo>-1 ) showDetails(ti, "*");
}


void showDetails(TabInfo *master_ti, char *master_field_value)
{
	int x, i;
	char query[MAXQUERY], buf[255];
	SQL_ROW row;
	SQL_RES *res;
	GtkListStore  *store;
	GtkTreeIter iter;

	TabInfo *ti_detail=((Params *)(master_ti->params))->TabInfoStructs[master_ti->DetailTableNo];
	ldebug(2,"showDetails entered: Master=%s, Detail=%s\n", master_ti->table, ti_detail->table);
	ldebug(2,"showDetails: Detail key_field=%s\n", ti_detail->fields[ti_detail->sortkey].name);
	
	gtk_notebook_set_current_page((GtkNotebook *)((Params *)(ti_detail->params))->detailsnotebook, ti_detail->notebook_page_num);
	
	if( !strcmp(master_field_value,"*") ) /* show all values */
		snprintf(query,sizeof(query) - 1,"select * from %s order by %s", ti_detail->table, ti_detail->fields[ti_detail->sortkey].name);
	else
		snprintf(query,sizeof(query) - 1,"select * from %s where %s='%s' order by %s", ti_detail->table, master_ti->master_field_name, master_field_value, ti_detail->fields[ti_detail->sortkey].name);
		
	ldebug(2,"showDetails: query:%s-\n", query);
	if( executeQuery( (Params *)(ti_detail->params), query) ) {
		showSqlError( (Params *)(ti_detail->params),"showDetails: Query in showRecords failed !");
		return;
	}
	res = sql_store_result( ((Params *)(ti_detail->params))->dbsock);

	// gtktreeview'da gosterilmek uzere bir gtk_list_store olusturulmaya baslaniyor
	// gtk_list_store'un field'larinin sayisi ve tipleri tanimlaniyor
	store = gtk_list_store_newv(ti_detail->numflds, ti_detail->field_types_for_gtk_list_store);
	
	ti_detail->nrrows=sql_num_rows(res);
	ldebug(2, "showDetails: nrrows=%d\n", ti_detail->nrrows);
	for(x=0; x < ti_detail->nrrows; x++) {
		if( (row = sql_fetch_row(res)) == NULL) {
			showSqlError( (Params *)(ti_detail->params),"showDetails: Error reading record %d !",x);
			break;
		}
		gtk_list_store_append(store, &iter);
		for(i=0; i < ti_detail->nrcols; i++) {
			if( ti_detail->field_types_for_gtk_list_store[i]==G_TYPE_INT ) {
				sprintf(buf, "%d", (int)row[i]);
				gtk_list_store_set(store, &iter, i, buf,-1);
			}
			else gtk_list_store_set(store, &iter, i, row[i],-1);
		}
	}
	free(res);
	
	//gtk_signal_connect( GTK_OBJECT(lookup_widget(ti->mainwindow,GtkTreeViewName)), "row-activated", GTK_SIGNAL_FUNC(ti->clist_line_selected_function), (gpointer)ti);
	
  	gtk_tree_view_set_model(ti_detail->gtktreeview, GTK_TREE_MODEL(store));
	//gtk_tree_view_set_model(ti->gtktreeview, NULL);
  	g_object_unref(store); // destroy model automatically with view
}

char *addStringQuotes(int type)
{
	if(
		type == FIELD_TYPE_DATE       || type == FIELD_TYPE_TIME   ||
		type == FIELD_TYPE_DATETIME   || type == FIELD_TYPE_STRING ||
		type == FIELD_TYPE_VAR_STRING
      )
		return "'";
	else return "";
}
