#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>
#include <syslog.h>

# include <gtk/gtk.h>
#include "mysqlstuff.h" /* Currently assumes a MYSQL database */

//#define DEBUG /* MACRO to make debugging easier */
//#define DEBUGLEVEL 4 /* only for ldebug calls */

#define SELECT  "SELECT * FROM %s "
#define INSERT  "INSERT INTO %s VALUES ("
#define UPDATE  "UPDATE %s SET "
#define DELETE  "DELETE FROM %s "
#define WHERE   " WHERE "
#define EQUAL   " = "
#define LIKE    " LIKE "
#define AND     " AND "
#define ORDERBY " ORDER BY "

#define MAXLOGSIZE 50000
#define MAXQUERY   10000
#define MAXMESSAGE 1024
#define MAXQUERY_LENGTH 2048


#define NR_TABLES 7  /* Number of the tables */

#define CONFIG_FILE "/etc/trackerd.conf"

struct ayr {
	char MTAHost[51];
	char FromName[51];
	char To1[51];
	char To2[51];
	char To3[51];
    char Subject[51];
	int Period;
    int TrackerRate;
	char Tracking[4];
	char LastUpdateTime[17];
	char SnmpRO[11];
    char SnmpRW[11];
    char sqlhost[127];
    char sqluser[20];
    char sqlpass[20];
	char sqldb[20];
    char ldaphost[250];
    char ldapbase[250];
    char ldapmanager[25];
    char ldapmanagerpass[250];
};


/* Just fof info, form mysql.h */
//typedef struct st_mysql_field {
//  char *name;                   /* Name of column */
//  char *table;                  /* Table of column if column was a field */
//  char *org_table;              /* Org table name if table was an alias */
//  char *db;                     /* Database for table */
//  char *def;                    /* Default value (set by mysql_list_fields) */
//  unsigned long length;         /* Width of column */
//  unsigned long max_length;     /* Max width of selected set */
//  unsigned int flags;           /* Div flags */
//  unsigned int decimals;        /* Number of decimals in field */
//  enum enum_field_types type;   /* Type of field. Se mysql_com.h for types */
//} MYSQL_FIELD;


/* contains info for one single browser panel (tab) */
typedef struct _TabInfo
{
	char		*table;       /* currently selected table */
	int			numflds;      /* number of dbfields (without opt. indices) */
	int			sortkey;      /* number of field we'll use to sort */
	int			numrecs;
	SQL_FIELD	*fields;      /* stores fields for current table */
	
	GType *field_types_for_gtk_list_store;
	/* stores field types for use with gtk_list_store_newv() function */
	/* when creating a new GtkListStore. */
	/* Determined by examining the fields of the table */
   
	int			nrcols;       /* number of columns (often fields) in browser */
	int			nrrows;       /* number of rows (records) in browser */
	GtkWindow	*mainwindow;

	GtkTreeView *gtktreeview;		/* pointer to clist widget in notebook */
	GtkWidget *frame;				/* GtkTreeView */
	
	guint notebook_page_num;		/* Holds the notebook_page_number of that ti */
	short int DetailTableNo;		/* Holds the detail table number of that master table*/
   									/* -1 means there is no detail table */
	char master_field_name[255];	/* master-detail relation field name */
	short int master_field_no;		/* field number of the master_field, this is set by the*/
									/* void set_master_detail_info(TabInfo *ti, char *master_field_name) function */
	 
   /* What to do when a record/line selected on the tab: call this function */
   int (*gtktreeview_line_selected_function)(GtkTreeView *treeview, gpointer user_data);
   
   int show_details; 	/* gtktreeview'a baglanmis "key-press-event"'i tarafindan kontrol ediliyor  */
   						/* ayni tusa pes pese iki kere basildiginda 1<->0 arasinda gecis yapiyor */
						/* Bu deger master tablonun secilen kayidinin detay tablosundaki bilgilerinin */
						/* scroll esnasinda gosterilip gosterilmeyecegini belirlemek icin kullaniliyor*/
						/* Bu deger record_selected() function'i tarafindan kullaniliyor */
   
   gchar **titles;
   int selectedrow;   /* number of selected row (or -1 if none) */
   
   void *params;
} TabInfo;


/* contains all global application info */
typedef struct _Params
{
	SQL       *dbsock;      /* MYSQL style : int dbsock for MSQL */
	GtkWindow *mainwindow;
	GtkWidget *notebook;	/* Main notebook */
	GtkWidget *detailsnotebook;	/* Detail notebook */
	GtkWidget *detailwin;
	GtkWidget *detailframe;
	
	char user[100];
	char pwd[100];
	char server[100];
	char database[100];
    
    TabInfo *TabInfoStructs[NR_TABLES];
	TabInfo *last_selected_gtktreeview; /* When a line of a gtktreeview is selected, that variable is set */
	/* This is used to determine which gtktreeview had the focus lastly */
	/* When a notebook page has been switched, this is set again, because user may click on the insert button without selecting */
	/* any line on the gtktreeview */
} Params;


/* main.c : */
void setSortKey(int row);

/* startup.c */
void trackerdbguiInit(Params *p, int argc, char *argv[]);
void storeWidgets(GtkWindow *main, Params *p);
void connectMainWindowSignals(GtkWindow *main, Params *p);
void set_master_detail_info(TabInfo *ti, char *master_field_name);
void on_help_usage(GtkMenuItem *menuitem, gpointer user_data);
void on_help_about(GtkMenuItem *menuitem, gpointer user_data);

/* actions.c */
void on_edit_show(GtkMenuItem *menuitem, gpointer user_data);
void on_edit_delete(GtkMenuItem *menuitem, gpointer user_data);
void on_edit_insert(GtkMenuItem *menuitem, gpointer user_data);
void on_edit_search(GtkMenuItem *menuitem, gpointer user_data);
void on_search_field_edited(GtkCellRendererText *celltext, const gchar *string_path, const gchar *new_text, gpointer data);


/* dialog.c : */
void ldebug(int level, char *format, ...);
void showVarArgMessage(Params *par, char *s,...);
void showMessage(char *s, Params *par);
void setLogWindow(GtkWidget *logwindow);
void showSqlError(Params *par, char *s, ...);
void writeLog(char *s,...);
void saveLogAction(GtkWidget *w, gpointer user_data );
GtkWidget* create_confirmbox(char *message);
GtkWidget* create_messagebox(char *message);
void ShowMessageBox(char *message, Params *params);
int ShowConfirmBox(char *question, Params *params);

/* helpers.c : */
int  executeQuery(Params *params, char *query);
int  getNumRecords(TabInfo *ti);

/* sqlconnect.c : */
int get_NumRecords_FieldNames(TabInfo *ti, SQL_RES *res);

/* optionactions.c : */
void on_database_activate(GtkMenuItem *item, gpointer  user_data);
void on_table_activate(GtkMenuItem *item, gpointer  user_data);

/* showrecords.c : */
void showRecords(TabInfo *ti );
void showRecordsTab(TabInfo *ti, SQL_RES *res);
void showDetails(TabInfo *master_ti, char *master_field_value);
void record_selected_showdetail(GtkWidget *clist, gint row, gint column, GdkEventButton *event, gpointer user_data );
int record_selected(GtkTreeView *treeview, gpointer user_data);
gboolean gtktreeview_onButtonPressed(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
int insert_key_pressed(TabInfo *ti);
char *addStringQuotes(int type);
void update_statusbar(Params *par, char *format, ...);
void on_DetailsGtkCheckButton_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 
/* showfields.c : */
int  getSortKey(TabInfo *ti);
void showFields(Params *p);

/* structhandlers.c : */
TabInfo *newTabInfostruct(Params *p, char *tablename, char *GtkTreeViewName);
void destroyTabInfostruct(TabInfo *ti);
void on_field_edited(GtkCellRendererText *celltext, const gchar *string_path, const gchar *new_text, gpointer data);
void set_master_detail_relation(Params *par, char *master_table_name, char *detail_table_name, char *master_field_name);

/* read_config.c */
int config_oku(struct ayr *ayarlar);
